#ifndef MOO_APP_H
#define MOO_APP_H

#include <QDir>
#include <QUrl>
#include <QStringList>
#include <QApplication>
#include <moo/types.h>

class QFile;
class QSettings;
class QCloseEvent;

namespace moo {

class App : public QApplication {
    Q_OBJECT

public:
    App(int &argc, char **argv,
        const QString &orgDomain,
        const QString &appName,
        const QString &appVersion);
    ~App();

    bool makeUserDataDir(const QString &name = QString());
    QString getUserDataDir(const QString &name = QString());
    QString getUserDataFile(const QString &name);
    QString saveTempFile(const QString &contents, QString &error, const QString &name = QString());
    bool openTempFile(QFile &file, QString &error, const QString &name = QString());
    QString tempFileName(const QString &name = QString());
    QString fileNameInTempDir(const QString &name);
    QDir makeTempDir();

    QSettings *settings(SettingsType type);

    void allWindowsClosed();

public Q_SLOTS:
    virtual void maybeQuit();
    virtual void aboutDialog() = 0;
    virtual void prefsDialog() = 0;

private:
    bool event(QEvent *e);
    void maybeQuit(QCloseEvent *e);
    virtual void loadFile(const QString &filename) = 0;
    QString topTempDir();
};

#define mooApp (static_cast<moo::App*>(QCoreApplication::instance()))

} // namespace moo

#endif // MOO_APP_H
